local _G = GLOBAL
local V3 = _G.Vector3
local TUNING = _G.TUNING

TUNING.VEST_MOD_DURABILITY_MULTI  = 2--1.5
TUNING.VEST_MOD_INSULATION_ADDON = 0 
TUNING.VEST_MOD_DAPPERNESS_MULTI = 1
TUNING.VEST_MOD_INSULATION_MULTI = 2

local function overwrite(tabula, name, func, post)
	if type(tabula) ~= "table" then return end 
	local old = tabula[name]
	tabula[name] = function(...)
		local unpack = unpack or table.unpack or _G.unpack
		
		local results_ante = {}
		if type(func) == "function" then results_ante = {func(...)} end 
		if #results_ante > 0 then return unpack(results_ante) end 
		
		local results_original = {}
		if type(old) == "function" then results_original = {old(...)} end 
		
		local results_post = {}
		if type(post) == "function" then results_post = {post(...)} end 
		if #results_post > 0 then return unpack(results_post) end 
		
		return unpack(results_original)
	end 
end 

local thatch = {anim = "ui_thatchpack_1x4", pos = {}}
for y = 0, 3 do 
	local pos = V3(-162 +(75 / 2), -y * 75 + 114 ,0)
	table.insert(thatch.pos, pos) 
end 

local icepack = {anim = "ui_icepack_2x3", pos = {}}
for y = 0, 2 do
	table.insert(icepack.pos, V3(-162, -y * 75 + 75 ,0))
	table.insert(icepack.pos, V3(-162 +75, -y * 75 + 75 ,0))
end

local function onopenfn(inst)
	inst.SoundEmitter:PlaySound("dontstarve/wilson/backpack_open", "open")
end 

local function onclosefn(inst)
	inst.SoundEmitter:PlaySound("dontstarve/wilson/backpack_close", "open")
end 

local function onburnt(inst)
	if inst.components.container then 
		inst.components.container:DropEverything()
		inst.components.container:Close()
		inst:RemoveComponent"container" 
	end 
end 

local function onequipfn(inst, owner)
	owner.components.inventory:SetOverflow(inst) 
	inst.components.container:Open(owner)
end 

local function onunequipfn(inst, owner)
	owner.components.inventory:SetOverflow(nil)
	inst.components.container:Close(owner)
end 

local function MakeContainer(inst, data)
	if inst.components.container then return print("vest already has container component", inst) end 
	inst:AddComponent"container"
	local container = inst.components.container 
	container:SetNumSlots(#data.pos)
	container.widgetslotpos = data.pos 
	container.widgetanimbank = data.anim 
	container.widgetanimbuild = data.anim 
	container.widgetpos = V3(-5,-70,0)
	container.side_widget = true 
	container.type = "pack"
	container.canbeopened = false 
	
	if not inst.SoundEmitter then inst.entity:AddSoundEmitter() end 
	container.onopenfn = onopenfn
	container.onclosefn = onclosefn
	
	inst:ListenForEvent("onremove", onburnt)-- using the same function fr now 
	overwrite(inst.components.burnable, "onburnt", onburnt) 
	overwrite(inst.components.equippable, "onequipfn", onequipfn)	
	overwrite(inst.components.equippable, "onunequipfn", onunequipfn)
	
	inst:AddTag"backpack"
--	inst.components.inventoryitem.cangoincontainer = false
end 

TUNING.TARSUIT_PERISHTIME = TUNING.RAINCOAT_PERISHTIME --/ TUNING.VEST_MOD_DURABILITY_MODIFIER

for vest, large in pairs{
	sweatervest = false, -- dapper, 60 insu
	
	trunkvest_summer = true, -- breezy, 60 insu
	trunkvest_winter = true, -- puffy, 240 insu 
	beargervest = true, -- bearger, 240 insu
	
	blubbersuit = false, -- bluuber, 240 winter ins + 100 wetness resist 
	tarsuit = false, -- looks kinda spacious doens't it? , full
	armor_snakeskin = false, -- snakeskin, 60 winter insu 
	raincoat = false, 
	
	armor_windbreaker = false, 
	
	reflectivevest = false, -- summer frest , 120 insu
	hawaiianshirt = false, -- hawaiian, 240 insu 
} do 
	AddPrefabPostInit(vest, function(inst)
	
		MakeContainer(inst, large and icepack or thatch) 
		
		if inst.components.fueled then 
			inst.components.fueled:InitializeFuelLevel(
				inst.components.fueled.maxfuel * TUNING.VEST_MOD_DURABILITY_MULTI)
		end 
		
		if inst.components.perishable then 
			inst.components.perishable:SetPerishTime(
				inst.components.perishable.perishtime * TUNING.VEST_MOD_DURABILITY_MULTI)
		end 
	
		if inst.components.insulator then 
			inst.components.insulator.insulation = inst.components.insulator.insulation 
				* TUNING.VEST_MOD_INSULATION_MULTI + TUNING.VEST_MOD_INSULATION_ADDON
		end 
		
		if inst.components.equippable and inst.components.equippable.dapperness > 0 then 
			inst.components.equippable.dapperness = 
				inst.components.equippable.dapperness * TUNING.VEST_MOD_DAPPERNESS_MULTI
		end 
	end)
end 
